/*
    EZLeser - a paystub reader (Einzahlungsschein Leser)

    Copyright (C) 2008  David Mosberger-Tang <dmosberger@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/
#define _GNU_SOURCE
#include <libgen.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <sys/stat.h>

int
main (int argc, char **argv)
{
  char *tmp_filename, *comment, *value, *line = NULL, *dir, *dev = NULL;
  char *output_file_name = NULL;
  FILE *mfp, *pfp, *ofp;
  struct stat stbuf;
  char buf[1024];
  ssize_t len;
  size_t size;

  tmp_filename = tempnam (NULL, "ez");

  if (argc > 1)
    {
      output_file_name = argv[1];
      snprintf (buf, sizeof (buf), "%s.xml", tmp_filename);
      ofp = fopen (buf, "w");
      if (!ofp)
	{
	  perror (buf);
	  exit (-1);
	}
    }
  else
    ofp = stdout;

  if (argc > 2)
    dev = argv[2];

  dir = dirname (argv[0]);

  snprintf (buf, sizeof (buf),
	    "/usr/bin/scanimage%s%s --mode color --resolution 300 | "
	    "%s/ppmautorot > %s.pbm",
	    dev ? " -d" : "", dev ? dev : "",
	    dir, tmp_filename);
  fprintf (stderr, "executing `%s'\n", buf);
  system (buf);

  snprintf (buf, sizeof (buf), "%s.pbm", tmp_filename);
  if (stat (buf, &stbuf) < 0 || stbuf.st_size == 0)
    {
      fprintf (ofp, "Fehler whrend dem Einlesen des Einzahlungsscheins.\n");
      fprintf (ofp, "Konnte das Bild nicht Einscannen.\n");
      unlink (buf);
      exit (-1);
    }

  snprintf (buf, sizeof (buf),
	    "/usr/bin/ppm2tiff -c lzw %s.pbm %s.tiff",
	    tmp_filename, tmp_filename);
  fprintf (stderr, "executing `%s'\n", buf);
  system (buf);

  snprintf (buf, sizeof (buf), "%s.pbm", tmp_filename);
  unlink (buf);

  snprintf (buf, sizeof (buf), "%s.tiff", tmp_filename);
  if (stat (buf, &stbuf) < 0 || stbuf.st_size == 0)
    {
      fprintf (ofp, "Fehler whrend dem Einlesen des Einzahlungsscheins.\n");
      fprintf (ofp,
	       "Konnte das Bild nicht auf das TIFF Format konvertieren.\n");
      unlink (buf);
      exit (-1);
    }

  snprintf (buf, sizeof (buf),
	    "TESSDATA_PREFIX=%s/../ %s/tesseract %s.tiff %s -l ocr",
	    dir, dir, tmp_filename, tmp_filename);
  fprintf (stderr, "executing `%s'\n", buf);
  system (buf);

  snprintf (buf, sizeof (buf), "%s.tiff", tmp_filename);
  unlink (buf);

  snprintf (buf, sizeof (buf), "%s/extract_fields %s.txt", dir, tmp_filename);
  fprintf (stderr, "opening pipe `%s'\n", buf);
  pfp = popen (buf, "r");
  if (!pfp)
    {
      fprintf (ofp, "Fehler whrend dem Einlesen des Einzahlungsscheins.\n");
      fprintf (ofp, "Konnte die Felder nicht Extrahieren.\n");
      exit (-1);
    }

  /* point of no return: try not to fail once the XML header has gone out: */

  fprintf (ofp, "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n");
  fprintf (ofp, "<group>\n");

  snprintf (buf, sizeof (buf), "%s/../map", dir);
  mfp = fopen (buf, "r");
  if (mfp)
    {
      while ((len = getline (&line, &size, mfp)) >= 0)
	{
	  if (len <= 1)
	    continue;
	  line[--len] = '\0';

	  comment = strchr (line, '#');
	  if (comment == line)
	    continue;
	  if (comment && comment[-1] != '\\')
	    *comment = '\0';

	  value = strchr (line, '=');
	  if (!value)
	    continue;
	  *value = '\0';
	  ++value;
	  fprintf (ofp, " <map name=\"%s\">%s</map>\n", line, value);
	}
      fclose (mfp);
    }

  while ((len = getline (&line, &size, pfp)) >= 0)
    {
      if (len <= 1)
	continue;
      line[--len] = '\0';
      value = strchr (line, '=');
      if (!value)
	continue;
      *value = '\0';
      ++value;
      fprintf (ofp, " <val name=\"%s\">%s</val>\n", line, value);
    }

  pclose (pfp);

  snprintf (buf, sizeof (buf), "%s.txt", tmp_filename);
  unlink (buf);

  fprintf (ofp, "</group>\n");
  fclose (ofp);

  /* atomically make the output-file appear: */
  if (output_file_name)
    {
      snprintf (buf, sizeof (buf), "%s.xml", tmp_filename);
      fprintf (stderr, "renaming %s to %s\n", buf, output_file_name);
      rename (buf, output_file_name);
    }
  return 0;
}
